#include "Movable.h"
#include "Actor.h"
#include "Man.h"
#include "Stage.h"
#include "Sprite.h"
// #include "Chars.h"
#include "ScanKeys.h"
#include "Item.h"
#include "Sound.h"
#include "VVram.h"

static const byte[] Chars = {
    // left
    0x1f, 0xaa, 0xa3, 0xa9,
    0x80, 0x20, 0x83, 0xaa,
    0x80, 0x20, 0xa3, 0xaa,
    // right
    0xa9, 0x1f, 0xaa, 0xa2,
    0x20, 0x1f, 0xa9, 0x82,
    0x20, 0x1f, 0xa9, 0xa2,
};
constexpr ptr<byte> pLeftChars = Chars + 4 * 3 * 0;
constexpr ptr<byte> pRightChars = Chars + 4 * 3 * 1;

Actor Man;

static void Show()
{
    ptr<byte> pChars;
    if (Man.dx == 0) {
        pChars = Man.pChars + 2 * 4;
    }
    else {
        pChars = Man.pChars + ((Man.status & Actor_SeqMask) << 2);
    }
    ShowSprite(&Man._, pChars);
}

void InitMan()
{
    byte b;
    b = pStage->manPosition;
    Man._.x = ToX(b) << CoordShift;
    Man._.y = (ToY(b) + 1) << CoordShift;
    Man._.sprite = Sprite_Man;
    Man.status = Actor_Live;
    Man.dx = 0;
    Man.pChars = pRightChars;
    Show();
}


void MoveMan()
{
    if (IsOnGrid(&Man._)) {
        sbyte dx;
        byte key;
        dx = 0;
        key = ScanKeys();
        if ((key & Keys_Left) != 0) {
            dx = -1;
            Man.pChars = pLeftChars;
        }
        else if ((key & Keys_Right) != 0) {
            dx = 1;
            Man.pChars = pRightChars;
        }
        if (dx != 0) {
            if (!CanMoveTo(&Man, dx)) {
                dx = 0;
            }
        }
        Man.dx = dx;
        HitItems();
    }
    if (Man.dx != 0) {
        byte seq;
        Man._.x += Man.dx;
        seq = ((Man._.x + CoordRate / 2) >> CoordShift) & 3;
        if (seq == 3) {
            seq = 1;
        }
        Man.status = (Man.status & ~Actor_SeqMask) | seq;
    }
    Show();
    ++Man._.clock;
}


void MoveManOnLift(ptr<Movable> pLift)
{
    if (MoveOnLift(&Man, pLift)) {
        Show();
    }
}


void FallMan()
{
    Fall(&Man);
    Show();
}


void HitMan(ptr<Movable> pMovable)
{
    if (IsNear(pMovable, &Man._)) {
        Man.status &= ~Actor_Live;
    }
}


void LooseMan()
{
    static const byte[] patterns = {
        0x80, 0x20, 0xa3, 0xaa,
        0x20, 0xa9, 0x80, 0xa2,
        0xab, 0xa3, 0x20, 0x80,
	    0xa2, 0x80, 0xac, 0x20,
    };
    byte i;
    i = 0;
    do {
        ShowSprite(&Man._, patterns + (i & 0xc));
        // UpdateSprites();
        Sound_Loose();
        DrawAll();
        i += 4;
    } while (i < 8 * 4);
    Man.status &= ~Actor_Live;
}
